@@ -24,6 +24,7 @@ import tempfile
 import time
 import uuid
 
+from cinderclient.v2 import client as cinderclient
 from os_brick.initiator import connector
 from oslo_concurrency import processutils
 from oslo_utils import units
@@ -37,7 +38,7 @@ from glance_store.tests.unit import test_store_capabilities
 
 class FakeObject(object):
     def __init__(self, **kwargs):
-        for name, value in six.iteritems(kwargs):
+        for name, value in kwargs.items():
             setattr(self, name, value)
 
 
@@ -75,6 +76,17 @@ class TestCinderStore(base.StoreBaseTest,
         self.assertIsNone(cc.client.auth_token)
         self.assertEqual('test_address', cc.client.management_url)
 
+    def test_get_cinderclient_with_user_overriden_and_region(self):
+        self.config(cinder_os_region_name='test_region')
+        fake_client = FakeObject(client=FakeObject(auth_token=None))
+        with mock.patch.object(cinderclient, 'Client',
+                               return_value=fake_client) as mock_client:
+            self.test_get_cinderclient_with_user_overriden()
+            mock_client.assert_called_once_with(
+                'test_user', 'test_password', 'test_project',
+                auth_url='test_address', cacert=None, insecure=False,
+                region_name='test_region', retries=3)
+
     def test_temporary_chown(self):
         class fake_stat(object):
             st_uid = 1
@@ -144,7 +156,7 @@ class TestCinderStore(base.StoreBaseTest,
             disconnect_volume=mock.Mock())
 
         @contextlib.contextmanager
-        def fake_chown(path):
+        def fake_chown(path, backend=None):
             yield
 
         def do_open():
@@ -158,12 +170,14 @@ class TestCinderStore(base.StoreBaseTest,
                              kwargs['conn'])
             return fake_connector
 
+        root_helper = "sudo glance-rootwrap /etc/glance/rootwrap.conf"
         with mock.patch.object(cinder.Store,
                                '_wait_volume_status',
                                return_value=fake_volume), \
                 mock.patch.object(cinder, 'temporary_chown',
                                   side_effect=fake_chown), \
-                mock.patch.object(cinder, 'get_root_helper'), \
+                mock.patch.object(cinder, 'get_root_helper',
+                                  return_value=root_helper), \
                 mock.patch.object(connector, 'get_connector_properties'), \
                 mock.patch.object(connector.InitiatorConnector, 'factory',
                                   side_effect=fake_factory):
@@ -177,7 +191,8 @@ class TestCinderStore(base.StoreBaseTest,
             fake_connector.disconnect_volume.assert_called_once_with(
                 mock.ANY, fake_devinfo)
             fake_volume.attach.assert_called_once_with(
-                None, None, attach_mode, host_name=socket.gethostname())
+                None, 'glance_store', attach_mode,
+                host_name=socket.gethostname())
             fake_volumes.detach.assert_called_once_with(fake_volume)
 
     def test_open_cinder_volume_rw(self):
